from __future__ import annotations

from dataclasses import asdict, dataclass

from pandas import DataFrame


def snake_to_camel_case(snake_str: str) -> str:
    return snake_str[0].lower() + snake_str.title()[1:].replace("_", "")


# TODO: Do not check for DataFrame instance here
def df_to_dataset(df: DataFrame = None) -> dict:
    return {"dataset": {"source": [df.columns.to_list()] + df.to_numpy().tolist()}}


def df_to_pie_data(df: DataFrame, name: str, value: str) -> list:
    return [{"name": row[0], "value": row[1]} for row in df[[name, value]].to_numpy()]


@dataclass
class BaseDataClass(object):
    def to_dict(self):
        return asdict(
            self,
            dict_factory=lambda x: {
                snake_to_camel_case(k): v for (k, v) in x if v is not None
            },
        )


class BaseOption(object):
    CHART_TYPE: str = None

    option = dict()

    def to_dict(self) -> dict:
        return self.option


class BaseOptionXY(BaseOption):
    def _create_series(self, x: str, y: str, series_options: dict = None) -> dict:
        return {
            "name": y,
            "type": self.CHART_TYPE,
            "encode": {"x": x, "y": y},
        } | (series_options or dict())

    def __init__(
        self,
        x: str = None,
        y: str = None,
        series_options: dict = None,
        data: DataFrame = None,
        **kwargs,
    ) -> None:
        self.x = x
        x_axis = dict(type="category") if self.CHART_TYPE == "bar" else dict()
        series = [self._create_series(x, y, series_options)]
        defaults = dict(xAxis=x_axis, yAxis=dict())
        if isinstance(data, DataFrame):
            defaults.update(df_to_dataset(data))

        self.option = defaults | kwargs | {"series": series}

    def add_series(self, y: str, series_options: dict = None) -> BaseOptionXY:
        self.option["series"].append(self._create_series(self.x, y, series_options))
        return self



from htmltools import HTMLDependency, Tag
from shiny import ui
from shiny.module import resolve_id

echarts_bindings_dep = HTMLDependency(
    "echarts-bindings",
    version="1.0.0",
    source={"package": "shinyecharts", "subdir": "srcjs"},
    script={"src": "echarts-bindings.js", "type": "module"},
    all_files=False,
)


def output_chart(id: str) -> Tag:
    return ui.div(
        echarts_bindings_dep,
        # Use resolve_id so that our component will work in a module
        id=resolve_id(id),
        class_="shiny-echarts-output",
        style="width: 600px;height:400px;",
    )


"""
def output_chart(id: str):
    pass
"""



from __future__ import annotations

from shinyecharts._core import BaseOption
from shinyecharts.options import ChartOption

# "boundaryGap": False,
# "splitLine": {"show": True},
# "axisTick": {"interval": 0.5},
# "minorTick": {},


class Line(BaseOption):
    """Line Option"""

    CHART_TYPE = "line"

    def __init__(self, x: str, y: str, data=None, **kwargs) -> None:
        self.data = data
        series = [{"name": y, "data": data[y].to_list(), "type": self.CHART_TYPE}]
        self.option = kwargs | {
            "x_axis": {
                "data": data[x].to_list(),
                "type": "category",
            },
            "series": series,
        }

    def add(self, y: str) -> Line_:
        self.option["series"].append(
            {"name": y, "data": self.data[y].to_list(), "type": self.CHART_TYPE}
        )
        return self

    def to_dict(self) -> dict:
        return ChartOption(**self.option).to_dict()



from __future__ import annotations

from dataclasses import dataclass, field

from pandas import DataFrame

from ._core import BaseDataClass, BaseOption, BaseOptionXY, df_to_pie_data


@dataclass
class ChartOption(BaseDataClass):
    """Chart option

    Note:
        See also [Option](https://echarts.apache.org/en/option.html).
    """

    title: dict = None
    legend: dict = None
    grid: dict = None
    x_axis: dict = field(default_factory=dict)
    y_axis: dict = field(default_factory=dict)
    polar: dict = None
    radius_axis: dict = None
    angle_axis: dict = None
    radar: dict = None
    data_zoom: list = None
    visual_map: list = None
    tooltip: dict = None
    # ...
    series: list = field(default_factory=list)


class Pie(BaseOption):
    """Pie Option"""

    CHART_TYPE = "pie"

    def __init__(
        self,
        series_options: dict = dict(),
        data: DataFrame | list = None,
        name: str = "name",
        value: str = "value",
        **kwargs,
    ) -> None:
        if isinstance(data, DataFrame):
            data = df_to_pie_data(data, name, value)

        self.series = [series_options | {"type": self.CHART_TYPE, "data": data}]
        self.option = (
            kwargs | {"x_axis": None, "y_axis": None} | {"series": self.series}
        )

    def to_dict(self) -> dict:
        return ChartOption(**self.option).to_dict()


class Line(BaseOptionXY):
    """Line Option"""

    CHART_TYPE = "line"


class Bar(BaseOptionXY):
    """Bar Option"""

    CHART_TYPE = "bar"


class Scatter(BaseOptionXY):
    """Scatter Option"""

    CHART_TYPE = "scatter"



from htmltools import Tag
from shiny.render.renderer import Jsonifiable, Renderer

from .chart import Chart
from .ui import output_chart


class ChartRenderer(Renderer[Chart]):
    """A decorator for a function that returns a chart"""

    def auto_output_ui(self) -> Tag:
        return output_chart(self.output_id)

    async def transform(self, value: Chart) -> Jsonifiable:
        return value.to_dict()
